#include <stdio.h>
#include <memory.h>

#ifdef _WIN32
#include <winsock2.h>
#else
#include <sys/types.h>
#include <sys/select.h>
#endif

#include <curl/curl.h>

int perform(CURLM * multi)
{
	int handles;
	CURLMcode code;
    fd_set fdread;
    fd_set fdwrite;
    fd_set fdexcep;
    int maxfd;
	CURLMsg * msg;
	int msgcount;

	for (;;)
	{
		code = curl_multi_perform(multi, &handles);
		if (0 == handles)
		{
			msg = curl_multi_info_read(multi, &msgcount);
			if (msg && (CURLMSG_DONE == msg->msg))
			{
				return CURLE_OK == msg->data.result;
			}
			return 0;
		}
		switch (code)
		{
			case CURLM_OK:
			{
				break;
			}
			case CURLM_CALL_MULTI_PERFORM:
			{
				continue;
			}
			default:
			{
				return 0;
			}
		}
		FD_ZERO(&fdread);
		FD_ZERO(&fdwrite);
		FD_ZERO(&fdexcep);
		curl_multi_fdset(multi, &fdread, &fdwrite, &fdexcep, &maxfd);
		select(maxfd + 1, &fdread, &fdwrite, &fdexcep, 0);
	}
}

size_t datasize = 0;
char data[500];

size_t WriteFunction(void *ptr, size_t size, size_t nmemb, void *stream)
{
	size_t rs = size * nmemb;
	memcpy(data + datasize, ptr, rs);
	datasize += rs;
	return nmemb;
}

int main(int argc, char **argv)
{
	CURLM * multi;
	CURL * easy;
	char * EXPECTED = "0192837465";

	curl_global_init(CURL_GLOBAL_ALL);
	multi = curl_multi_init();
	easy = curl_easy_init();

	curl_easy_setopt(easy, CURLOPT_WRITEFUNCTION, WriteFunction);
	curl_easy_setopt(easy, CURLOPT_RESUME_FROM, 10);
	curl_easy_setopt(easy, CURLOPT_URL, "file://test00.txt");

	curl_multi_add_handle(multi, easy);
	if (perform(multi))
		printf("OK: data retrieved successfully\n");
	else
		printf("ERROR: data retrieve failed\n");

	curl_multi_remove_handle(multi, easy);
	curl_easy_cleanup(easy);
	curl_multi_cleanup(multi);

	curl_global_cleanup();

	if (datasize == strlen(EXPECTED))
	{
		if (0 == memcmp(data, EXPECTED, strlen(EXPECTED)))
		{
			printf("OK: data is correct\n");
			return 0;
		}
	}

	printf("ERROR: data is incorrect\n");

	return 0;
}
